#! /bin/bash

# export UCF_FORCE_CONFFOLD=1 
# vserver-dist-upgrade.sh [ '-y -o Dpkg::Options::=--force-confdef -o Dpkg::Options::=--force-confold' ]


CHECK_SERVICES=${CHECK_SERVICES:-true}
RESTART_SERVICES=${RESTART_SERVICES:-false}
CHECK_ETC=${CHECK_ETC:-true}
set -u

do_install_squeeze_lts=true

VSERVER_CONFIG_DIR="/etc/vservers"
FORCE=no

export ABORT_CODE=99
export LANG=C
export LC_ALL=C


apt_command='/usr/bin/apt-get'
bash_command='/bin/bash'
test_command='/usr/bin/test'
grep_command='/bin/grep'
cat_command='/bin/cat'
ed_command='/bin/ed'

HOSTNAME=`/bin/hostname`
host=""

update_fehler_liste=""
upgrade_fehler_liste=""
started_services=""
interrupted=false
apt_args=()
vservers=""
#[ $# -gt 0 ] && args=$1

while [ $# -gt 0 ]; do
	case "$1" in
		--vforce)
			FORCE=yes
		;;
		--vhost)
			host=$HOSTNAME
		;;
		--squeeze-lts)
			do_install_squeeze_lts=true
		;;
		--no-squeeze-lts)
			do_install_squeeze_lts=false
		;;
		-*)
			apt_args+=("$1")
		;;
		*'['*|*\**)
			vservers+=' '`ls -d $VSERVER_CONFIG_DIR/$1 | sed "s#$VSERVER_CONFIG_DIR/##"`
		;;
		[a-z]*)
			vservers+="$1 "
		;;
		*)
			echo "Argumentfehler"
			exit1	
		;;
	esac
	shift
done


if $CHECK_SERVICES ; then
	services_before=`mktemp -t vserver-dist-upgrade.services_before.XXXX`
	services_after=`mktemp -t vserver-dist-upgrade.services_after.XXXX`
	services_diff=`mktemp -t vserver-dist-upgrade.services_diff.XXXX`
fi

endausgabe_func () {
        echo
        [ -n "$update_fehler_liste" ] && echo "Update-Fehler:$update_fehler_liste"
        [ -n "$upgrade_fehler_liste" ] && echo "Upgrade-Fehler:$upgrade_fehler_liste"
        [ -n "$started_services" ] && echo "Dienste neu gestartet: $started_services"
	if $CHECK_SERVICES ; then
		rm -f $services_before
		rm -f $services_after
		rm -f $services_diff
	fi
}

ask_cont () {
	echo
	echo -n "$v: "
	echo $*
        while read -p 'weiter?(Y|n) ' ; do
		REPLY=${REPLY:0:1}
                case "$REPLY" in
                        y|Y|j|J|'') 
                                return 0
                        ;;
                        n|N)
                                endausgabe_func
				exit
                        ;;
                esac
        done
}

int_func () {
	interrupted=true
	ask_cont "Unterbrochen durch Ctrl-C"
}

#vprefix_func () {
#	if [ -z "$v" ] ; then
#		echo 
#	else
#		echo "vserver $v exec"
#	fi
#}

search_running_services_func () {
	#vps  cft '?' | grep -v '\\_' | awk '{print $2,$NF}' | grep ^$context | awk '{print $NF}' | sort -u
	#netstat -xlnp  | `vprefix_func` bash | tail -n +3 | awk '{print $(NF-1)}' | cut -d'/' -f2 | sort -u
	EXCLUDE_SERVICES='logger screen bash smtp smtpd debian-start'
	local listen_services=`echo 'netstat -tulnp' | $vprefix $bash_command | tail -n +3 | awk '{print $7}' | cut -d'/' -f1 | grep -v -- '-' | grep -v '^$'`
	if [ -n "$listen_services" ] ; then
		listen_services=`$vprefix ps --no-heading -o comm $listen_services`
	else
		listen_services=""
	fi
	sort -u <(echo 'ps --no-heading cft?' | $vprefix $bash_command | grep -v '\\_' | awk '{print $NF}') <(echo "$listen_services") | 
	grep -v  '^-$' | sed 's/.pid$//' | grep -v '^[0-9]*$' | 
	while read s; do
		[ -z "$s" ] && continue
		for e in $EXCLUDE_SERVICES; do
			[ "$s" = "${e:0:15}" ] && continue 2
		done
		echo "$s"
	done

}

find_missing_services_func () {
	sleep 2
	search_running_services_func >$services_after
	diff $services_before $services_after >$services_diff
	missing_servs=`grep '^<' $services_diff | cut -d' ' -f2`
}

trap int_func SIGINT



install_squeeze_lts_source () {
	#printf "a\n%s\n.\nwq %s\n" 'deb    http://http.debian.net/debian/ squeeze-lts main contrib non-free' '/etc/apt/sources.list.d/squeeze-lts.list' | $vprefix $ed_command 
	#local ltsfile='/etc/apt/sources.list.d/squeeze-lts.list'
	local ltsfile='/etc/apt/sources.list'
	$vprefix $bash_command -c "printf '\n%s\n' 'deb    http://http.debian.net/debian/ squeeze-lts main contrib non-free' >>$ltsfile"
	local preferencesfile='/etc/apt/preferences'
	if $vprefix $test_command -f /etc/apt/preferences ; then
		$vprefix $bash_command -c "printf '\n\n%s\n%s\n%s\n' 'Package:  *' 'Pin:  release n=squeeze-lts' 'Pin-Priority:  510' >>$preferencesfile"
	fi
}

#[ -z "$vservers" ] && vservers=`cat $VSERVER_CONFIG_DIR/*/name`
#[ -z "$vservers" ] && vservers=`ls $VSERVER_CONFIG_DIR`
[ -z "$host" ] && vservers=`ls $VSERVER_CONFIG_DIR`
for v in $vservers $host; do 
	apt_command='/usr/bin/apt-get'
	vprefix=""
	if $test_command $v  = "$HOSTNAME" ; then
		echo "host upgrade !!"
	elif vserver $v running  ; then
		context=`$cat_command $VSERVER_CONFIG_DIR/$v/context`
	        [ -n "$v" ] && vprefix="vserver $v exec"
	else
		continue
	fi
		[ "`$vprefix $cat_command /etc/debian_version | cut -d. -f1`" = 5 ] && apt_command='/usr/bin/aptitude' 
		if $do_install_squeeze_lts && [ "`$vprefix $cat_command /etc/debian_version | cut -d. -f1`" = 6 ] && ! $vprefix $grep_command -q squeeze-lts /etc/apt/sources.list /etc/apt/sources.list.d/*.list ; then
			install_squeeze_lts_source 
			status=$?
                	if [ $status -ne 0 ] ; then
                        	update_fehler_liste+="  $v"
				if ! $interrupted; then
                        		ask_cont "install squeeze-lts-source  endet mit Fehler-Status $status"
				fi
                	fi
		fi
                if [ $FORCE = no ] && $vprefix $test_command -f /.NOUPDATE ; then
                        echo "$v: /.NOUPDATE exists, continue "
                        continue
                fi
		echo
		echo $v - update
		echo '--------' 
		$CHECK_ETC && $vprefix $bash_command -c "type check-etc-rcs-files" && $vprefix check-etc-rcs-files --before-dist-upgrade
		[ $? -eq $ABORT_CODE ] && ask_cont
		interrupted=false
		#vserver $v exec aptitude update
		$vprefix $apt_command update
                status=$?
                if [ $status -ne 0 ] ; then
                        update_fehler_liste+="  $v"
			if ! $interrupted; then
                        	ask_cont "aptitude update endet mit Fehler-Status $status"
			fi
                fi
		$vprefix $apt_command -V -y  install lihas-keyring
		interrupted=false
		echo $v - upgrade
		echo '========' 
		$CHECK_SERVICES && search_running_services_func >$services_before
		#export UCF_FORCE_CONFFOLD=1 # nicht immer richtig
		#  vserver $v exec aptitude -V -o Dpkg::Options::="--force-confdef" -o Dpkg::Options::="--force-confold"  "${apt_args[@]}"  dist-upgrade
		if [ ${#apt_args[*]} -eq 0 ]; then
			$vprefix  $apt_command -V dist-upgrade
		else
			$vprefix  $apt_command -V "${apt_args[@]:-}" dist-upgrade
		fi
                status=$?
                if [ $status -ne 0 ] ; then
                        upgrade_fehler_liste+="  $v"
			if ! $interrupted; then
                        	ask_cont "$apt_command dist-upgrade endet mit Fehler-Status $status"
			fi
                fi
		interrupted=false
		$CHECK_ETC && $vprefix  $bash_command -c "type check-etc-rcs-files" && $vprefix  check-etc-rcs-files --after-dist-upgrade
		[ $? -eq $ABORT_CODE ] && ask_cont
		#vserver $v exec aptitude clean
		$vprefix  $apt_command clean
		if $CHECK_SERVICES ; then 
			find_missing_services_func
			if [ -n "$missing_servs" ] ; then
				if $RESTART_SERVICES; then
					for s in $missing_servs; do
						[ "$s" = master ] && s=postfix
						if $vprefix  $test_command -x /etc/init.d/$s  ; then
							$vprefix  /etc/init.d/$s start 
						elif s=${s%-*}; $vprefix $test_command -x /etc/init.d/$s  ; then
							$vprefix  /etc/init.d/$s start
						fi
						started_services+="$v:$s "
					done
					find_missing_services_func
				fi
				if [ -n "$missing_servs" ] ; then
					ask_cont "gestoppte Dienste nach dem Upgrade: "$missing_servs
				fi
			fi
			new_servs=`grep '^>' $services_diff | cut -d' ' -f2` 
			if [ -n "$new_servs" ] ; then
				ask_cont "neu gestartete Dienste nach dem Upgrade: "$new_servs
			fi
		fi
done

endausgabe_func

